require "sinatra/base"
require "rack/ssl" unless ENV['RACK_ENV'] == "development"
require "base64"
require "addressable/uri"
require "databasedotcom-oauth2"
require "haml"

require_relative "model"

class ChaterPollApp < Sinatra::Base

  #-------------------------------------------
  # Environment variable check
  #-------------------------------------------
  REQUIRED_ENVIRONMENT_VARS = ["COOKIE_SECRET", 
    "SALESFORCE_KEY", 
    "SALESFORCE_SECRET", 
    "SALESFORCE_SANDBOX_KEY", 
    "SALESFORCE_SANDBOX_SECRET"]
  fail "The following environment variables must not be nil: #{REQUIRED_ENVIRONMENT_VARS.join(',')}" \
    if REQUIRED_ENVIRONMENT_VARS.collect{|var| ENV[var] }.include?(nil)

  #-------------------------------------------
  # Sinatra configuration                                                         
  #-------------------------------------------
  configure do
    enable :logging
    set :app_file            , __FILE__
    set :root                , File.expand_path("../..",__FILE__)
    set :port                , ENV['PORT']
    set :raise_errors        , Proc.new { false }
    set :show_exceptions     , false
    set :token_encryption_key, Base64.strict_decode64(ENV['COOKIE_SECRET'])
    set :endpoints           , nil
    set :default_endpoint    , "login.salesforce.com"
  end

  settings.endpoints = {
    settings.default_endpoint => {
      :key      => ENV['SALESFORCE_KEY'], 
      :secret   => ENV['SALESFORCE_SECRET']}, 
    "test.salesforce.com" => {
      :key      => ENV['SALESFORCE_SANDBOX_KEY'],
      :secret   => ENV['SALESFORCE_SANDBOX_SECRET']}
  }
  settings.endpoints.default = endpoints[settings.default_endpoint]

  error do
    e = env['sinatra.error']
    title = "Unanticipated Error"
    description = ""
    if e
      title = title + ": " + e.class.to_s
      description = description + e.message
    end
    haml :error, :locals => {:title => title, :description => description}
  end

  before do
	  unless token.nil? || client.nil?
	    puts "token:\n#{token.inspect}"
	    puts "client:\n#{client.inspect}"
  	  client.debugging = true
    end
  end

  #-------------------------------------------
  # Rack stack
  #-------------------------------------------
  use Rack::SSL unless ENV['RACK_ENV'] == "development"
  use Rack::Session::Cookie, :expire_after => 60*60*7    # holds oauth2 token in encrypted, serialized form
  use Databasedotcom::OAuth2::WebServerFlow,
    :debugging            => true,
    :token_encryption_key => settings.token_encryption_key,
    :endpoints            => settings.endpoints, 
    :display              => "touch",
    :scope                => "api chatter_api id refresh_token"

  # mixes in client, me, authenticated?, etc.
  include Databasedotcom::OAuth2::Helpers
  
  #-------------------------------------------
  # Routes
  #-------------------------------------------
  get '/authenticate' do
	  if unauthenticated?
      haml :terms, :layout => :authenticate, :locals => { :url => '/auth/salesforce', :state => params[:state] }
    else
      redirect to(sanitize_state(params[:state])) 
    end
  end

  get '/unauthenticate' do
    request.env['rack.session'] = {}
    redirect to("/")
  end

  get '/auth/failure' do
    haml :error, :locals => {:title => "Authentication Error!", :description => params[:message]}
  end

  get '/polls/new' do
    authenticate!
    poll = Poll.new
    2.times.each{ |i| poll.choices << Choice.new }
    haml :polls_new, :locals => { :poll => poll, :url => '/polls/save' }
  end

  get '/polls/save' do
    authenticate!
    poll_params = params["poll"] || {}
    poll_params['organization_id'] = client.org_id
    poll_params['user_id']         = client.user_id
    poll_params['username']        = me.username
    poll_params['display_name']    = me.name
    poll_params['email']           = me.email
  	poll = Poll.new poll_params
    10.times do |i|
      break if i > 1 && params["answer#{i}"] == nil
      poll.choices_attributes=[{:value => params["answer#{i}"], :sequence => i}]
    end
  	if poll.valid?
    	poll.save
    	uri = Addressable::URI.new
      uri.query_values = {
        :text   => "\"#{poll.question}\" - respond to this poll question by " + 
                   "clicking here #{base_url}/polls/#{poll.id}",
        :retURL => "/polls/#{poll.id}" 
      }
    	redirect to("/posts/new?" + uri.query)
	  else
      haml :polls_new, :locals => { :poll => poll, :url => '/polls/save' }
    end
  end

  get '/polls/:poll_id' do
    authenticate!
    #filter by both id and organization_id
    poll = Poll.filter[ \
      :id              => params[:poll_id], 
      :organization_id => client.org_id    ]
    response = Response.filter[ \
      :poll_id => poll.id,
      :user_id => me.id       ] unless poll == nil
    if poll == nil
      haml :error, :locals => {:title => "Not found!", :description => "Record not found."}
    elsif response == nil
      haml :responses_new, :locals => { :poll => poll, :url => "/responses/save" }
    else
      haml :polls_view, :locals => { :poll => poll }
    end
  end

  get '/polls' do
    authenticate!
    polls = Poll.filter(:organization_id => client.org_id).order(:created_at.desc).all
    haml :polls, :locals => { :polls => polls }
  end


  get '/posts/new' do
    authenticate!
    haml :posts_new, :locals => { 
      :url => "/posts/save",
      :text   => params[:text], 
      :retURL => params[:retURL] 
    }
  end

  get '/posts/save' do
    authenticate!
    if params[:text] == nil || params[:text].empty?
      haml :posts_new, :locals => { 
        :url => "/posts/save",
        :text   => params[:text],
        :retURL => params[:retURL] 
      }
    else
      me.post_status(params[:text])
    	redirect to(params[:retURL] )
    end
  end

  get '/responses/save' do
    authenticate!
  	response = Response.new({
  	  :choice_id => params[:response][:choice_id], 
  	  :user_id   => me.id
  	})
  	choice = Choice[response.choice_id]
  	poll = Poll[choice.poll_id]
  	response.poll_id = poll.id
    poll.response_count = poll.response_count + 1 
    choice.response_count = choice.response_count + 1 
    DB.transaction do 
      poll.save
      choice.save
    	response.save
    end
  	redirect to("/polls/#{poll.id}")
  end

  get '/me' do
    authenticate!
    haml :me 
  end
  
  get '/terms' do
    haml :terms 
  end
  
  get '/*' do
    authenticate!
    polls = Poll.filter(:organization_id => me.organization_id).order(:created_at.desc).all
    haml :polls, :locals => { :polls => polls }
  end
  
  #-------------------------------------------
  # Helpers
  #-------------------------------------------
  helpers do
  	def authenticate!
  	  if unauthenticated?
        uri = Addressable::URI.new
        uri.query_values = {:state => request.fullpath}
    	  redirect to("/authenticate?#{uri.query}") 
  	  end
  	end

    def sanitize_state(state = nil)
      state = "/" if state.nil? || state.strip.empty?
      state
    end
    
    def base_url
      @base_url ||= "#{request.env['rack.url_scheme']}://#{request.env['HTTP_HOST']}"
    end

  end

end


